; docformat = 'rst'
;+
; This is an example program to demonstrate how to create a latent heat plot from NASA HDF5 files
; with Coyote Graphics routines.
;
; :Categories:
;    Graphics
;    
; :Examples:
;    Save the program as "latent_heat_plot.pro" and run it like this::
;       IDL> .RUN latent_heat_plot
;       
; :Author:
;    FANNING SOFTWARE CONSULTING::
;       David W. Fanning 
;       1645 Sheely Drive
;       Fort Collins, CO 80526 USA
;       Phone: 970-221-0438
;       E-mail: david@idlcoyote.com
;       Coyote's Guide to IDL Programming: http://www.idlcoyote.com
;
; :History:
;     Change History::
;        Written, 29 January 2013 by David W. Fanning.
;
; :Copyright:
;     Copyright (c) 2013, Fanning Software Consulting, Inc.
;-
PRO Latent_Heat_Plot
 
   ; Read the HDF5 data file from NASA Surface Turbulent Fluxes
   ; database: http://disc.sci.gsfc.nasa.gov/datacollection/GSSTF_3.html.
   file = 'GSSTF.2b.2008.01.01.he5'
   
   ; This is an HDF5 file. The data name that we want to plot is "E".
   fileID = H5F_Open(file)
   dataName = '/HDFEOS/GRIDS/SET1/Data Fields/E'
   dataID = H5D_Open(fileID, dataName)
   dataspaceID = H5D_Get_Space(dataID)
   dims = H5S_Get_Simple_Extent_Dims(dataspaceID)
   lon_dim = dims[0]
   lat_dim = dims[1]
   latentHeat = H5D_Read(dataID)
   fill_valueID = H5A_Open_Name(dataID, '_FillValue')
   fill_value = H5A_Read(fill_valueID)
   H5A_Close, fill_valueID
   H5D_Close, dataID    
   
   ; The next line is critical to finding all the missing or fill values.
   fill_value = fill_value[0]
   
   ; Find the missing indices and set to NaN.
   missingIndices = Where(latentHeat EQ fill_value, missingCnt)
   IF missingCnt GT 0 THEN latentHeat[missingIndices] = !Values.F_NAN
   
   ; Calculate the contour levels.
   nlevels = 8
   minData = Floor(Min(latentHeat, MAX=maxData, /NaN))
   levels = [minData, Indgen(nlevels)*50]
   
   ; Create X and Y vectors in longitude and latitude. This is
   ; necessary for laying contours on a map.
   lon = cgScaleVector(Findgen(lon_dim),-180,179) + 0.5
   lat = cgScaleVector(Findgen(lat_dim), -90, 89) + 0.5
   
   ; Open a graphics window.
   cgDisplay
   
   ; Load display colors.
   cgLoadCT, 25, /Brewer, NColors=10, Bottom=1, /Reverse
   
   ; Set up the map projection and overlay the contour plot.
   cgMap_Set, Position=[0.05,0.05,0.95,0.75]
   cgContour, latentHeat, lon, lat, /Overplot, /Cell_Fill, $
       C_Colors=Indgen(10)+1, Levels=levels
   cgContour, latentHeat, lon, lat, /Overplot, Levels=levels
   
   ; Annotate the map.
   cgMap_Continents, Color='tan', /Fill
   cgMap_Continents, Color='black'
   cgMap_Grid, /Box_Axes

   ; Set up a discrete color bar.
   labels = ['< 0','0-50', '50-100', '100-150', '150-200', '200-250', $
             '250-300', '300-350', '350-400', '>400']
   cgDCBar, NColors=10, Bottom=1, Color='black', $
      Position=[0.05, 0.88, 0.95, 0.92], Rotate=-45, $
      Labels=labels, Charsize=charsize, $
      Title='Surface Latent Heat Flux (watts/meter$\up2$)'

END ;*****************************************************************

; This main program shows how to call the program and produce
; various types of output.

  ; Display the plot in a graphics window.
  Latent_Heat_Plot
  
  ; Display the plot in a resizeable graphics window.
  cgWindow, 'Latent_Heat_Plot', WTitle='Latent Heat Plot in a Resizeable Graphics Window'
  
  ; Create a PostScript file.
  cgPS_Open, 'latent_heat_plot.ps'
  Latent_Heat_Plot
  cgPS_Close
  
  ; Create a PNG file with a width of 600 pixels.
  cgPS2Raster, 'latent_heat_plot.ps', /PNG, Width=600

END